#include "wx/wx.h"
#include <wx/glcanvas.h>
#include <GL/glu.h>


class MyApp : public wxApp
{
public:
    virtual bool OnInit();
};

class TestGLCanvas;


class MyFrame : public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
            const wxSize& size, long style = wxDEFAULT_FRAME_STYLE);

    void OnExit(wxCommandEvent& event);

    TestGLCanvas* m_canvas;

    DECLARE_EVENT_TABLE()
};


class TestGLCanvas: public wxGLCanvas
{
  friend class MyFrame;
public:
 TestGLCanvas(wxWindow *parent, const wxWindowID id = -1, 
    const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
    long style = 0, const wxString& name = "TestGLCanvas");
   
 ~TestGLCanvas(void);

 void OnPaint(wxPaintEvent& event);
 void OnSize(wxSizeEvent& event);
 void OnEraseBackground(wxEraseEvent& event);
 void OnEnterWindow( wxMouseEvent& event );
 
 void Render( void );
 void InitGL(void);
   
private:
  bool   m_init;


DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(TestGLCanvas, wxGLCanvas)
  EVT_SIZE(TestGLCanvas::OnSize)
  EVT_PAINT(TestGLCanvas::OnPaint)
  EVT_ERASE_BACKGROUND(TestGLCanvas::OnEraseBackground)
  EVT_ENTER_WINDOW( TestGLCanvas::OnEnterWindow )
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
END_EVENT_TABLE()



IMPLEMENT_APP(MyApp)


static void Reshape(int width, int height)
{

  glViewport(0,0,width,height); 
  glMatrixMode( GL_PROJECTION ); 
  glLoadIdentity(); 
  glFrustum(-10,10,-10,10,1.8,7);
  glMatrixMode( GL_MODELVIEW ); 
  glLoadIdentity();

}



bool MyApp::OnInit()
{

  MyFrame *frame = new MyFrame(NULL, "Exemplo Basico", wxPoint(50, 50),
                               wxSize(350, 350));

  wxMenu *winMenu = new wxMenu;

  winMenu->Append(wxID_EXIT, "&Fechar");

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(winMenu, "&Janela");

  frame->SetMenuBar(menuBar);

  frame->m_canvas = new TestGLCanvas(frame, -1, wxPoint(0, 0), wxSize(500, 500));

  frame->Show(TRUE);
  
  return TRUE;
}


MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                 const wxSize& size, long style) 
         : wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = NULL;
}

TestGLCanvas::TestGLCanvas(wxWindow *parent, wxWindowID id,
    const wxPoint& pos, const wxSize& size, long style, const wxString& name):
  wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name )
{
    m_init = FALSE;
}


TestGLCanvas::~TestGLCanvas()
{
}

void MyFrame::OnExit(wxCommandEvent& event)
{
    Destroy();
}

void TestGLCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

void TestGLCanvas::OnEraseBackground(wxEraseEvent& event)
{
}

void TestGLCanvas::OnPaint( wxPaintEvent& event )
{
    Render();
}

void TestGLCanvas::Render()
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

    SetCurrent();

    if (!m_init)
    {
        InitGL();
        m_init = TRUE;
    }
    
  glClearColor(0.0f,0.0f,0.0f,0.0f); 
  glClear( GL_COLOR_BUFFER_BIT ); 


       glColor3f(0.2, 0.2, 0.2);

        // Draw the sides of the three-sided pyramid
        glBegin(GL_TRIANGLE_FAN);
                glVertex3d(0, 3, 0);
                glColor3f(0.3, 0.4, 0.5);
                glVertex3d(0, -3, -3);
                glVertex3d(-3, -3, 3);
                glVertex3d(3, -3, 3);
                glVertex3d(0, -3, -3);
        glEnd();

       
        glColor3f(0.2, 0.3, 0.5);

        // Draw the base of the pyramid
        glBegin(GL_TRIANGLES);
                glVertex3d(0, -3, -3);
                glVertex3d(-3, -3, 3);
                glVertex3d(3, -3, 3);
        glEnd();

       glLoadIdentity();
       glRotatef(-30, 1.0, 0.0, 0.0);
       glRotatef(30, 0.0, 1.0, 0.0);
       glRotatef(30, 0.0, 0.0, 1.0);

       glTranslatef(-4.0,0.0,-4.0);

       glColor3f(1.0,0.5,0.5);
        // Draw the sides of the three-sided pyramid
        glBegin(GL_TRIANGLE_FAN);
                glVertex3d(0, 3, 0);
                glColor3f(1.0, 0.4, 0.5);
                glVertex3d(0, -3, -3);
                glVertex3d(-3, -3, 3);
                glVertex3d(3, -3, 3);
                glVertex3d(0, -3, -3);
        glEnd();

       
        glColor3f(0.2, 0.3, 0.5);

        // Draw the base of the pyramid
        glBegin(GL_TRIANGLES);
                glVertex3d(0, -3, -3);
                glVertex3d(-3, -3, 3);
                glVertex3d(3, -3, 3);
        glEnd();

  glFlush();
  SwapBuffers();
}

void TestGLCanvas::InitGL()
{
   SetCurrent();
   glClearColor(0.0, 0.0, 0.0, 0.0);
   

}

void TestGLCanvas::OnSize(wxSizeEvent& event)
{
    int width, height;
    GetClientSize(& width, & height);
    Reshape(width, height);
        if (height==0)  
	{ 
                height=1;   
	}  


#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
    }
}


